/*
 * Created on Apr 9, 2009
 *
 */
package qoppa.webViewer;

import java.awt.Window;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;

import com.qoppa.pdf.PDFException;
import com.qoppa.pdfViewer.IPDFOpener;
import com.qoppa.pdfViewer.PDFViewerBean;

public class HTTPOpenerText implements IPDFOpener
{
    private URL m_Context;
    private URL m_PDFListURL;
    private String m_ListDelimiters;
    
    /**
     * Create an HTTP opener that will retrieve a list of PDF files from a text file located at the given URL.
     * The list should contain one line per document, each line having two items, separated by the tab character.
     * 
     * @param context The base URL to use when the URL's to the PDF documents are relative URL's
     * @param pdfListText The list of PDF documents, in text format
     * @param delimiters The list of delimiters separating the name of the document from the URL that the document
     * will be read from
     */
    public HTTPOpenerText (URL context, URL pdfListText, String delimiters)
    {
        m_Context = context;
        m_PDFListURL = pdfListText;
        m_ListDelimiters = delimiters;
    }

    /**
     * Implementation of the openFile method to satisfy the IPDFOpener interface.  This implementation
     * shows a tree of documents to the user (read from the pdfListProvider URL), lets the user choose one
     * of the documents and then loads the PDF
     * 
     * @param viewerBean The PDFViewerBean that made the call
     */
    public void openFile(PDFViewerBean viewerBean) throws PDFException
    {
        try 
        {
            // Read all the text returned by the server
            BufferedReader in = new BufferedReader(new InputStreamReader(m_PDFListURL.openStream()));

            // Read the list of PDFs, separated by new lines
            Vector pdfList = new Vector();
            String str = in.readLine();
            while (str != null)
            {
                // Get the name and URL of the PDF document
                StringTokenizer tokens = new StringTokenizer (str, m_ListDelimiters);
                String pdfName = null;
                String pdfURL = null;
                if (tokens.hasMoreTokens())
                {
                    pdfName = tokens.nextToken();
                }
                if (tokens.hasMoreTokens())
                {
                    pdfURL = tokens.nextToken();
                }

                // If we have a name-URL pair, add it to the list
                if (pdfName != null && pdfURL != null)
                {
                    pdfList.add (new PDFNameAndURL (pdfName, pdfURL));
                }
                
                // Read the next line
                str = in.readLine();
            }
            in.close();
            
            // Check that we have a list
            if (pdfList.size() == 0)
            {
                viewerBean.showMessage("Server returned an empty list.");
            }
            
            // Create PDF tree
            DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode ();
            for (int count = 0; count < pdfList.size(); ++count)
            {
                rootNode.add(new DefaultMutableTreeNode (pdfList.get (count)));
            }
            
            // Show the list to the user
            Window parentWindow = SwingUtilities.windowForComponent(viewerBean);
            PDFNameAndURL chosePDF = (PDFNameAndURL)PDFListDialog.choosePDF(parentWindow, rootNode);
            if (chosePDF != null)
            {
                // Create URL object
                URL pdfURL = new URL (m_Context, chosePDF.m_PDFURL);

                // Load the document
                viewerBean.loadPDF (pdfURL);
            }
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
    }
}
