/*
 * Created on Apr 13, 2009
 *
 */
package qoppa.webViewer;

import java.awt.Window;
import java.io.BufferedInputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.Vector;

import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;

import nanoxml.XMLElement;

import com.qoppa.pdf.PDFException;
import com.qoppa.pdfViewer.IPDFOpener;
import com.qoppa.pdfViewer.PDFViewerBean;

public class HTTPOpenerXML implements IPDFOpener
{
    private URL m_Context;
    private URL m_PDFListURL;
    
    /**
     * Create an HTTP opener that will retrieve a list of PDF files from an XML file located at the given URL.
     * The structure of the XML file is described in the documentation.
     * 
     * @param context The base URL to use when the URL's to the PDF documents are relative URL's
     * @param pdfListXML The list of PDF documents, in text format
     */
    public HTTPOpenerXML (URL context, URL pdfListXML)
    {
        m_Context = context;
        m_PDFListURL = pdfListXML;
    }

    /**
     * Implementation of the openFile method to satisfy the IPDFOpener interface.  This implementation
     * shows a tree of documents to the user (read from the pdfListProvider URL), lets the user choose one
     * of the documents and then loads the PDF
     * 
     * @param viewerBean The PDFViewerBean that made the call
     */
    public void openFile(PDFViewerBean viewerBean) throws PDFException
    {
        try 
        {
            // Read the XML data
            XMLElement xmlPDF = new XMLElement ();
            InputStream inStream = new BufferedInputStream (m_PDFListURL.openStream());
            xmlPDF.parseFromInputStream(inStream);
            inStream.close();

            // Loop through children
            DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode ();
            addChildren (rootNode, xmlPDF.getMutableChildren());

            // Check that we have a list
            if (rootNode.getChildCount() == 0)
            {
                viewerBean.showMessage("Server returned an empty list.");
            }
            
            // Show the list to the user
            Window parentWindow = SwingUtilities.windowForComponent(viewerBean);
            PDFNameAndURL chosePDF = (PDFNameAndURL)PDFListDialog.choosePDF(parentWindow, rootNode);
            if (chosePDF != null)
            {
                // Create URL object
                URL pdfURL = new URL (m_Context, chosePDF.m_PDFURL);

                // Load the document
                viewerBean.loadPDF (pdfURL);
            }
        }
        catch (Throwable t) 
        {
            t.printStackTrace();
            viewerBean.showError("Error opening file: ", t);
        }
    }
    
    private void addChildren (DefaultMutableTreeNode parentNode, Vector children)
    {
        for (int index = 0; index < children.size(); ++index)
        {
            XMLElement child = (XMLElement)children.get (index);
            if ("PDF".equalsIgnoreCase(child.getName()))
            {
                PDFNameAndURL pdfInfo = new PDFNameAndURL (child.getStringAttribute("Name"), child.getStringAttribute("SourceURL"));
                parentNode.add(new DefaultMutableTreeNode (pdfInfo));
            }
            else if ("Folder".equalsIgnoreCase(child.getName()))
            {
                String folderName = child.getStringAttribute("Name");
                DefaultMutableTreeNode folderNode = new DefaultMutableTreeNode (folderName);
                addChildren(folderNode, child.getMutableChildren());
                parentNode.add (folderNode);
            }
        }
    }
}
