/*
 * Created on Apr 9, 2009
 *
 */
package qoppa.webViewer;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;

import com.qoppa.pdf.PDFException;

public class PDFListDialog extends JDialog implements ActionListener, MouseListener
{
    private JPanel jpContent = null;
    private JScrollPane jScrollPane = null;
    private JTree jtFileList = null;
    
    private Object m_Return;
    private JLabel jLabel = null;
    private JPanel jPanel = null;
    private JButton jbOk = null;
    private JButton jbCancel = null;

    private PDFListDialog (Frame owner)
    {
        super (owner);
		initialize();
    }

    private PDFListDialog (Dialog owner)
    {
        super (owner);
		initialize();
    }

    /**
     * This method initializes this
     * 
     */
    private void initialize() {
        this.setSize(new Dimension(344, 292));
        this.setTitle("Choose A Document");
        this.setModal(true);
        this.setContentPane(getJpContent());
    		
    }
    public static Object choosePDF (Window owner, TreeNode rootNode) throws PDFException
    {
        // Create the dialog
        PDFListDialog dialog;
        if (owner instanceof Frame)
        {
            dialog = new PDFListDialog ((Frame)owner);
        }
        else if (owner instanceof Dialog)
        {
            dialog = new PDFListDialog ((Dialog)owner);
        }
        else
        {
            throw new PDFException ("Invalid parent window: " + owner);
        }
        dialog.setLocationRelativeTo(owner);
        
        // Show root handles if there is sub-levels
        for (int count = 0; count < rootNode.getChildCount(); ++count)
        {
            if (rootNode.getChildAt(count).isLeaf() == false)
            {
                dialog.getJtFileList().setShowsRootHandles(true);
                break;
            }
        }
        
        // Set the tree model
        dialog.getJtFileList().setModel(new DefaultTreeModel (rootNode));
        
        // Show the dialog and wait
        dialog.setVisible(true);
        return dialog.m_Return;
    }

    /**
     * This method initializes jpContent	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJpContent()
    {
        if (jpContent == null)
        {
            BorderLayout borderLayout = new BorderLayout();
            borderLayout.setHgap(5);
            borderLayout.setVgap(5);
            jLabel = new JLabel();
            jLabel.setText("Please choose a document:");
            jpContent = new JPanel();
            jpContent.setLayout(borderLayout);
            jpContent.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));
            jpContent.add(jLabel, BorderLayout.NORTH);
            jpContent.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
            jpContent.add(getJPanel(), BorderLayout.SOUTH);
        }
        return jpContent;
    }
    /**
     * This method initializes jScrollPane	
     * 	
     * @return javax.swing.JScrollPane	
     */
    private JScrollPane getJScrollPane()
    {
        if (jScrollPane == null)
        {
            jScrollPane = new JScrollPane();
            jScrollPane.setViewportView(getJtFileList());
        }
        return jScrollPane;
    }
    /**
     * This method initializes jtFileList	
     * 	
     * @return javax.swing.JTree	
     */
    private JTree getJtFileList()
    {
        if (jtFileList == null)
        {
            jtFileList = new JTree();
            jtFileList.setRootVisible(false);
            jtFileList.setShowsRootHandles(false);
            jtFileList.addMouseListener(this);
        }
        return jtFileList;
    }
    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanel()
    {
        if (jPanel == null)
        {
            FlowLayout flowLayout = new FlowLayout();
            flowLayout.setAlignment(FlowLayout.RIGHT);
            jPanel = new JPanel();
            jPanel.setLayout(flowLayout);
            jPanel.add(getJbOk(), null);
            jPanel.add(getJbCancel(), null);
        }
        return jPanel;
    }
    /**
     * This method initializes jbOk	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getJbOk()
    {
        if (jbOk == null)
        {
            jbOk = new JButton();
            jbOk.setText("Ok");
            jbOk.setPreferredSize(new Dimension(75, 26));
            jbOk.addActionListener(this);
        }
        return jbOk;
    }
    /**
     * This method initializes jbCancel	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getJbCancel()
    {
        if (jbCancel == null)
        {
            jbCancel = new JButton();
            jbCancel.setText("Cancel");
            jbCancel.setPreferredSize(new Dimension(75, 26));
            jbCancel.addActionListener(this);
        }
        return jbCancel;
    }
    
    public void actionPerformed(ActionEvent e)
    {
        if (e.getSource() == getJbOk())
        {
            DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)getJtFileList().getLastSelectedPathComponent();
            if (selectedNode == null || selectedNode.isLeaf() == false)
            {
                JOptionPane.showMessageDialog(this, "Must select one document.");
                return;
            }
            
            m_Return = selectedNode.getUserObject();
            dispose();
        }
        else if (e.getSource() == getJbCancel())
        {
            m_Return = null;
            dispose();
        }
    }
    
    public void mouseClicked(MouseEvent e){}
    public void mouseEntered(MouseEvent e){}
    public void mouseExited(MouseEvent e){}
    public void mouseReleased(MouseEvent e){}
    public void mousePressed(MouseEvent e)
    {
        if (e.getSource() == getJtFileList() && e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() == 2)
        {
            DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)getJtFileList().getLastSelectedPathComponent();
            if (selectedNode != null && selectedNode.isLeaf())
            {
                m_Return = selectedNode.getUserObject();
                dispose();
            }
        }
    }
}  //  @jve:decl-index=0:visual-constraint="10,10"
