/*
 * Created on Apr 9, 2009
 *
 */
package qoppa.webViewer;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.security.AccessController;

import javax.swing.JApplet;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.ProgressMonitorInputStream;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

import nanoxml.XMLElement;

import com.qoppa.pdf.PDFException;
import com.qoppa.pdfViewer.PDFViewerBean;

public class PDFWebViewer extends JApplet
{
    private JPanel jpContentPane;
    private JPanel jpTitlePane;
    private PDFViewerBean m_ViewerBean = null;

    // Constants to determine true and false
    public final static String STRING_FALSE = "False";
    public final static String NUMBER_FALSE = "0";
    public final static String STRING_TRUE = "True";
    public final static String NUMBER_TRUE = "1";
    
    // Show progress
    private boolean m_ShowProgress = true;

    /**
     * Initialize the applet.
     */
    public void init() 
    {
    	// Set the system look and feel
    	try
    	{
    		UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    	}
    	catch (Exception e)
    	{
    	}
    	
    	// Create components
        this.setContentPane(createContentPane());
        
        // Config using XML configuration file
    	configXML ();
    }
    
    public void start()
    {
        // Set open button visibility according to the BrowseAllowed flag.
        getPDFViewerBean().getToolbar().getjbOpen().setVisible (toBoolean (getParameter("OpenVisible"), true, true));

        // Set print button visibility according to the PrintAllowed flag.
        getPDFViewerBean().getToolbar().getjbPrint().setVisible (toBoolean (getParameter ("PrintVisible"), true, true));
        
        // Set the scale
        String scale = getParameter ("Scale");
        if (scale != null && scale.trim().length() > 0)
        {
        	getPDFViewerBean().getInitialViewSettings().setMagnification(scale);
        	getPDFViewerBean().getInitialViewSettings().setOverrideMagnification(true);
        }

        // Check if there is a custom opener and install it
        installCustomOpener();

        // Load the documents in a separate thread so the
        // applet can finish starting and update the GUI
        Thread loadThread = new Thread(new Runnable() {
        	public void run()
        	{
        		// Load a PDF document from a URL
                final String urlString = getParameter("OpenURL");
                if (urlString != null && urlString.trim ().length() > 0)
                {
                    privLoadPDF(urlString);
                    getPDFViewerBean().setScale2D(100);
                }
        	}
        });
        loadThread.start();
    }
    
    private void configXML ()
    {
    	// Get the jPDFViewer component
    	PDFViewerBean viewerBean = getPDFViewerBean();

        // Read stamps from XML file
        String configURLParam = getParameter ("Config");
        if (configURLParam != null && configURLParam.trim().length() != 0)
        {
            try
            {
                // Form URL object
                URL configURL = createURL (configURLParam);
                
                // Load the XML Tree
                URLConnection urlConnect = configURL.openConnection();
                urlConnect.setUseCaches(false);
                InputStream xmlStream = urlConnect.getInputStream();
                
                XMLElement configXML = new XMLElement();
                configXML.parseFromInputStream(xmlStream);
                xmlStream.close();
                
                for (int count = 0; count < configXML.countChildren(); ++count)
                {
                    XMLElement child = configXML.getChild(count);
                    if (child.getName().equalsIgnoreCase("Toolbar"))
                    {
                        configToolbar (child, viewerBean);
                    }
                    else if (child.getName().equalsIgnoreCase("Key"))
                    {
                        setKey (child.getContent());
                    }
                    else if (child.getName().equalsIgnoreCase("Options"))
                    {
                        setGeneralOptions(child);
                    }
                }
            }
            catch (Throwable t)
            {
                System.err.println ("Error loading stamps configuration: " + configURLParam);
                t.printStackTrace();
            }
        }
    }
    
    private void setKey (String key)
    {
        if (key != null && key.length() > 0)
        {
            key = key.trim();
            boolean valid = PDFViewerBean.setKey(key);
            if (valid == false)
            {
                valid = PDFViewerBean.setAppletKey(key, this);
            }

            // Echo invalid key
            if (valid == false)
            {
                System.out.println ("Invalid Key: " + key);
                System.out.println ("Host Name: " + getDocumentBase().getHost().toLowerCase());
            }
        }
    }
    
    private URL createURL (String urlString) throws MalformedURLException
    {
    	if (urlString == null || urlString.trim().length() == 0)
    	{
    		return null;
    	}

        if (urlString.startsWith("http://") || urlString.startsWith ("https://"))
        {
            return new URL (urlString);
        }
        else
        {
            return new URL (getDocumentBase(), urlString);
        }
    }

    public void stop()
    {
        // Clear any document that is loaded
        getPDFViewerBean().clearDocument();
    }
    
    private JPanel createContentPane()
    {
        if (jpContentPane == null) {
            jpContentPane = new JPanel();
            jpContentPane.setLayout(new BorderLayout ());
            jpContentPane.setBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.LOWERED));
            jpContentPane.add(getjpTitlePane(), BorderLayout.NORTH);
            jpContentPane.add(getPDFViewerBean(), BorderLayout.CENTER);
        }
        return jpContentPane;
    }
    
    private JPanel getjpTitlePane()
    {
        if (jpTitlePane == null)
        {
            jpTitlePane = new JPanel();
            jpTitlePane.setLayout(new BorderLayout());
            jpTitlePane.setBorder(new EmptyBorder(6, 0, 8, 0));
            JLabel title = new JLabel ("Qoppa Software - " + PDFViewerBean.getVersion());
            title.setHorizontalAlignment(SwingConstants.CENTER);
            title.setFont(new Font ("sansserif", Font.BOLD, 14));
            jpTitlePane.add(title, BorderLayout.CENTER);
        }
        return jpTitlePane;
    }
    
    public PDFViewerBean getPDFViewerBean() 
    {
        if (m_ViewerBean == null) 
        {
            m_ViewerBean = new PDFViewerBean();
            m_ViewerBean.setName("PDFViewerBean");
        }
        return m_ViewerBean;
    }
    
    private boolean toBoolean (String str, boolean emptyValue, boolean defaultValue)
    {
        // empty value
        if (str == null || str.trim().length() == 0)
        {
            return emptyValue;   
        }

        // Check for true
        if (str.equalsIgnoreCase(STRING_TRUE) || str.equalsIgnoreCase(NUMBER_TRUE))
        {
            return true;
        }
        else if (str.equalsIgnoreCase(STRING_FALSE) || str.equalsIgnoreCase(NUMBER_FALSE))
        {
            return false;
        }
        else
        {
            return defaultValue;
        }
    }
    
    /**
     * Loads a PDF document into the PDFViewerBean.  This method loads the PDF document
     * inside a privileged action so that it can be called from Javascript.  Otherwise,
     * the method would throw a security exception because Javascript does not get much
     * permissions to start with.
     * 
     * @param pdfURLString The URL to the PDF document.
     * 
     * @return The number of pages in the document
     */
    public int loadPDF (final String pdfURLString)
    {
        // We need to enclose this in a privileged action so that it
        // can execute properly when called from Javascript
		Object rc = AccessController.doPrivileged(new java.security.PrivilegedAction() {
			public Object run()
			{
				int pageCount = privLoadPDF(pdfURLString);
				return Integer.valueOf(pageCount);
			}
		});

		if (rc != null && rc instanceof Integer)
		{
			return ((Integer)rc).intValue();
		}
		return 0;
	}

    /**
     * Internal implementation of the load PDF method.  This method is safe
     * to call from the applet context, but not from Javascript.  If calling
     * from Javascript, then loadPDF() should be used instead.
     * 
     * @param pdfURLString The URL to the PDF document.
     * 
     * @return The number of pages in the document
     */
    private int privLoadPDF (String pdfURLString)
    {
        try
        {
            // Form URL object
            URL pdfURL = new URL (getCodeBase(), pdfURLString);
            
            // Get the URLConnection
            URLConnection urlConnect = pdfURL.openConnection();
            urlConnect.setUseCaches(false);
            
            // Input stream to the URL
            InputStream inputStream = urlConnect.getInputStream();
            
            // Set up the ProgressMonitorInputStream
            if (m_ShowProgress)
            {
            	ProgressMonitorInputStream pmInputStream = new ProgressMonitorInputStream(getPDFViewerBean(), "Loading document...", inputStream);
            	pmInputStream.getProgressMonitor().setMaximum(urlConnect.getContentLength());
            	inputStream = pmInputStream;
            }
            
            // Load the PDF
            getPDFViewerBean().loadPDF(inputStream);
            
            // Optional call to allow keystroke navigation (up/down arrows, page up/down, ...)
            getPDFViewerBean().getRootPane().getContentPane().requestFocus();
            
            // Return the page count
            return getPDFViewerBean().getPageCount();
        }
        catch (MalformedURLException mURL)
        {
            displayError ("Invalid PDF URL: " + mURL.getMessage());
        }
        catch (Throwable t)
        {
            if (t.getCause() instanceof PDFException)
            {
                displayError (t.getCause().getMessage());
            }
            else
            {
                displayError ("Error loading PDF: " + t.getMessage());
            }
        }
        return 0;
    }
    
    private void displayError (String errorMsg)
    {
        JOptionPane.showMessageDialog(this, errorMsg);
    }
    
    private void installCustomOpener()
    {
        // Get parameters in HTML
        String textListOpenerURL = getParameter ("DocumentList_Text");
        String xmlListOpenerURL = getParameter("DocumentList");
        
        // check for XML based opener
        if (xmlListOpenerURL != null && xmlListOpenerURL.trim().length() > 0)
        {
            try
            {
                URL pdfListURL = new URL (getCodeBase(), xmlListOpenerURL);
                getPDFViewerBean().setPDFOpener(new HTTPOpenerXML (getCodeBase(), pdfListURL));
            }
            catch (MalformedURLException badURL)
            {
                displayError("Invalid opener URL: " + xmlListOpenerURL);
            }
        }
        
        // Check for text list based opener
        else if (textListOpenerURL != null && textListOpenerURL.trim().length() > 0)
        {
            try
            {
                URL pdfListURL = new URL (getCodeBase(), textListOpenerURL);
                getPDFViewerBean().setPDFOpener(new HTTPOpenerText (getCodeBase(), pdfListURL, "\t"));
            }
            catch (MalformedURLException badURL)
            {
                displayError("Invalid opener URL: " + textListOpenerURL);
            }
        }
    }

    private void configToolbar (XMLElement toolbarXML, PDFViewerBean notesBean) throws IOException
    {
        // Parse the XML tree
        for (int count = 0; count < toolbarXML.countChildren(); ++count)
        {
            XMLElement child = toolbarXML.getChild(count);
            if (child.getName().equalsIgnoreCase("RemoveButton"))
            {
                String methName = child.getStringAttribute("MethodName");
                if (methName != null && methName.length() > 0)
                {
                    boolean found = removeToolbarButton (methName, m_ViewerBean.getToolbar());
                    if (found == false)
                    {
                    	found = removeToolbarButton (methName, m_ViewerBean.getSelectToolbar());
                    }
                    
                    if (found == false)
                    {
                        System.err.println ("Did not find toolbar button to remove: " + methName);
                    }
                }
            }
            else if (child.getName().equalsIgnoreCase("HideToolbar"))
            {
            	String toolbarName = child.getStringAttribute("Name");
            	if (toolbarName != null)
            	{
            		toolbarName = toolbarName.trim();
            		if ("navigation".equalsIgnoreCase(toolbarName))
            		{
            			m_ViewerBean.getToolbar().setVisible (false);
            		}
            		else if ("selection".equalsIgnoreCase(toolbarName))
            		{
            			m_ViewerBean.getSelectToolbar().setVisible(false);
            		}
            	}
            }
        }
    }
    
    private void setGeneralOptions(XMLElement options)
    {
        for (int count = 0; count < options.countChildren(); ++count)
        {
            XMLElement child = options.getChild(count);
            if ("HideTitle".equalsIgnoreCase(child.getName()))
            {
                getjpTitlePane().setVisible(false);
            }
            else if ("HideProgress".equalsIgnoreCase(child.getName()))
            {
            	m_ShowProgress = false;
            }
        }
    }
    
    private boolean removeToolbarButton (String methName, JToolBar toolbar)
    {
        try
        {
        	System.out.println (methName);
            Method m = toolbar.getClass().getMethod(methName, null);
            if (m != null)
            {
                Object button = m.invoke(toolbar, null);
                if (button instanceof Component)
                {
                    toolbar.remove((Component)button);
                    return true;
                }
            }
        }
        catch (NoSuchMethodException nsme)
        {
        	System.err.println (methName + ": Not a valid method.");
        }
        catch (Throwable t)
        {
            t.printStackTrace();
        }
        
        return false;
    }
}
